;-----------------------------------------------------------------------------
;                     Bootmanager Inside File System MBR
;-----------------------------------------------------------------------------
; This boot sector is used when installing the bootmanager as a file in the
; root of an existing file system. The code segment will replace the MBR of
; the system harddrive, and therefore executed each time the system boot. The
; boot sector needs to be configured with the number of the partition on which
; the bootmanager exist. After locating the right partition, it will load the
; boot sector, calculate where the root directory start, search for the boot-
; manager file, when located then read it to memory.
;
; Due to the size limitations of the MBR sector, the BIFS code must be file
; system specific. For that reason this file have a code segment for each file
; system it supports.





        org     0100h

        mov     word [CurrentSegment], es

        mov     eax, 00003h
        int     010h

        mov     ebp, ErrorCode.100

;-----------------------------------------------------------------------------
; Get drive parameters. These will be save for later use.
        mov     eax, 00800h                ; DL should contain the current drive
        mov     edx, 080h
        int     013h
        jc      near FatalError

        inc     dh
        mov     byte [Max_Head], dh

        push    cx
        and     cx, 0003Fh
        mov     byte [Max_Sector], cl

        pop     cx
        and     cx, 0FFC0h
        xchg    ch, cl
        rol     ch, 002h
        inc     cx
        mov     word [Max_Cylinder], cx




;-----------------------------------------------------------------------------
; Get boot sector of partition
;-----------------------------------------------------------------------------
; Now we will begin searching for the rest of the bootmanager. We will start
; with the MBR table on the drive we booted from.

        mov     eax, 00201h      ; Function: Load one sector from drive
        mov     ebx, Address.BootSector
        mov     cx, [Partition.CylSec]
        mov     dh, [Partition.Head]
        mov     dl, [Partition.Drive]
        int     013h
        jc      near FatalError

; Since we got here the disk transfer went well. Now we will first check the
; signature.
        cmp     word [Address.BootSector+01FEh], 0AA55h
        jnz     near FatalError



;-----------------------------------------------------------------------------
; Doing some calculations.
;-----------------------------------------------------------------------------
; First we are going to calculate the LBA address for the first sector of the
; partition. SI should point to that entry.

CalculateAddresses:
        mov     cx, [Partition.CylSec]
        mov     dh, [Partition.Head]
        mov     dl, [Partition.Drive]

        call    ConvertCHSToLBA

        mov     dword [PartitionStartLBA], eax

; Now we got the LBA address of our partition. Next we are going to calculate
; the LBA address of our first FAT. That should be:
; Start + Hidden Sector
        movzx   eax, word [Address.BootSector+00Eh]
        add     eax, [PartitionStartLBA]
        mov     dword [FATStartLBA], eax

; And then the LBA address of the root directory.
        movzx   eax, word [Address.BootSector+016h]
        mov     cl, [Address.BootSector+010h]
        mul     cl
        add     eax, [FATStartLBA]
        mov     dword [RootDirectoryStartLBA], eax

; Finally calculate the LBA address of the first cluster.
        movzx   eax, word [Address.BootSector+011h]
        xor     edx, edx
        movzx   ecx, byte [RootDirectory.BytesPerEntry]
        mul     ecx
        xor     edx, edx
        movzx   ecx, word [Address.BootSector+00Bh]
        div     ecx
        add     eax, [RootDirectoryStartLBA]
        mov     dword [DataStartLBA], eax

; Then we load the first part of the root directory into memory.
SearchRootDirectory:
        mov     eax, [RootDirectoryStartLBA]
        call    ConvertLBAToCHS
        mov     eax, 00220h      ; Function: Load one sector from drive
        mov     ebx, Address.RootDirectory
        mov     dl, 080h
        int     013h
        jc      near FatalError

        mov     edi, Address.RootDirectory
        movzx   ecx, word [Address.BootSector+011h]
.LocateBootmanagerFile_Loop1:
        push    ecx
        push    edi
        mov     esi, BootmanagerFileName
        mov     ecx, 00Bh
        rep     cmpsb
        je      .FoundIt
        pop     edi
        add     edi, 020h
        pop     ecx
        loopnz  .LocateBootmanagerFile_Loop1
        add     esp, 00Ch
        jmp     near FatalError

.FoundIt:
        pop     edi
        pop     ecx
        movzx   eax, word [edi+01Ah]
        mov     dword [CurrentCluster], eax
        mov     eax, dword [edi+01Ch]
        push    eax
        movzx   eax, word [Address.BootSector+00Bh]
        movzx   ecx, byte [Address.BootSector+00Dh]
        mul     ecx
        mov     ecx, eax
        xor     edx, edx
        pop     eax
        div     ecx
        inc     eax
        mov     ecx, eax



LoadFileIntoMemory:
.Loop1:
.CalculateLBAFromCluster:
        push    ecx

        mov     eax, [CurrentCluster]
        sub     eax, 002h
        movzx   ecx, byte [Address.BootSector+00Dh]
        mul     ecx
        add     eax, [DataStartLBA]

        call    ConvertLBAToCHS



.LoadClusterIntoMemory:
        mov     es, [Program.Segment]
        mov     eax, 00200h      ; Function: Load one sector from drive
        mov     al, [Address.BootSector+00Dh]
        mov     ebx, [Program.Pointer]
        mov     dl, [Partition.Drive]
        int     013h
        jc      near FatalError
        mov     es, [CurrentSegment]

        movzx   eax, word [Address.BootSector+00Bh]
        movzx   ecx, byte [Address.BootSector+00Dh]
        mul     ecx
        add     dword [Program.Pointer], eax



.IsThereMoreClusters:
        pop     ecx
        dec     ecx
        jz      near LoadFileIntoMemory.Exit
        push    ecx



.IsClusterInFATArea:
        mov     eax, [CurrentCluster]
        cmp     eax, [FAT_Area.FirstEntry]
        jb      .LoadFATArea_Previous
        cmp     eax, [FAT_Area.LastEntry]
        ja      .LoadFATArea_Next



.ValidateCluster:
        mov     eax, [CurrentCluster]
        sub     eax, [FAT_Area.FirstEntry]
        mul     dword [FAT_Area.BytesPerEntry]
        add     eax, Address.FAT_Area
        cmp     word [eax], 00000h
        jz      near .InvalidCluster
        cmp     word [eax], 0FFFFh
        jz      .EndOfFile
        cmp     word [eax], 0FFF8h
        jae     .InvalidCluster
        cmp     word [eax], 0FFF6h
        jae     .BadCluster
        movzx   ebx, word [eax]
        mov     dword [CurrentCluster], ebx

        pop     ecx
        jmp     .Loop1



.LoadFATArea_Previous:
.LoadFATArea_Next:
        mov     eax, [CurrentCluster]
        mul     dword [FAT_Area.BytesPerEntry]
        xor     edx, edx
        movzx   ecx, word [Address.BootSector+00Bh]
        div     ecx
        push    eax
        add     eax, [FATStartLBA]

        call    ConvertLBAToCHS

        mov     eax, 00200h      ; Function: Load one sector from drive
        mov     al, FAT_Area.SizeInSectors
        mov     ebx, Address.FAT_Area
        mov     dl, [Partition.Drive]
        int     013h
        jc      near FatalError

        pop     eax
        xor     edx, edx
        movzx   ecx, word [Address.BootSector+00Bh]
        mul     ecx
        xor     edx, edx
        div     dword [FAT_Area.BytesPerEntry]
        mov     dword [FAT_Area.FirstEntry], eax
        add     eax, [FAT_Area.NumberOfEntries]
        mov     dword [FAT_Area.LastEntry], eax

        jmp     .ValidateCluster



.BadCluster:
.EndOfFile:
.InvalidCluster:
.Exit:
        int3
        movzx   ebx, byte [Partition.Drive]
        mov     dh, byte [Partition.Drive]
        mov     dl, byte [Partition.Drive]
        jmp     09000h:00200h
        int     020h



















FatalError:
        int3
        push    ebp                     ; Saving error pointer.

        mov     eax, 000001301h         ; Write string to screen function
        mov     ebx, 000000007h         ; Color attribute
        mov     ecx, Text.PressAKeyToReboot_Length
        mov     edx, 000000100h
        mov     ebp, Text.PressAKeyToReboot
        int     010h

        mov     eax, 000001301h         ; Write string to screen function
        mov     ebx, 000000007h         ; Color attribute
        mov     ecx, Text.ErrorCode_Length
        mov     edx, 000000200h
        mov     ebp, Text.ErrorCode
        int     010h

        mov     eax, 000001301h         ; Write string to screen function
        mov     ebx, 000000007h         ; Color attribute
        mov     ecx, 000000003h         ; Fixed string length
        mov     edx, 000000208h
        pop     ebp
        int     010h

        mov     eax, 000000000h         ; Waiting for a keypress
        int     016h

        mov     ax, 000401h             ; And the reset
        mov     ds, ax
        mov     word [0072h], 1234h
        jmp     0FFFFh:00000h




;-----------------------------------------------------------------------------
; Convert CHS to LBA
;-----------------------------------------------------------------------------
; This routine will convert the CHS addresses to LBA addresses. CX should
; contain the Cylinder / Sector information in the packed format, and DH
; should contain the Head information.

ConvertCHSToLBA:
        push    ecx
        push    edx
        and     cx, 0FFC0h
        xchg    ch, cl
        rol     ch, 002h
        movzx   eax, cx

        mul     dword [Max_Head]

        pop     edx
        and     dx, 0FF00h
        movzx   edx, dh
        add     eax, edx

        mul     dword [Max_Sector]

        pop     ecx
        and     cl, 03Fh
        dec     cl
        movzx   ecx, cl
        add     eax, ecx

        ret




;-----------------------------------------------------------------------------
; Convert LBA to CHS
;-----------------------------------------------------------------------------
; This routine will convert the CHS addresses to LBA addresses. CX should
; contain the Cylinder / Sector information in the packed format, and DH
; should contain the Head information.

ConvertLBAToCHS:
        xor     edx, edx
        div     dword [Max_Sector]
        inc     dl
        movzx   ecx, dl

        xor     edx, edx
        div     dword [Max_Head]
        xchg    dl, dh

        xchg    al, ah
        ror     al, 002h

        or      ecx, eax

        ret




;-----------------------------------------------------------------------------
; Text Strings.
;-----------------------------------------------------------------------------

Text:
.ErrorCode:                     db      'Error: '
.ErrorCode_Length:              equ     $-.ErrorCode
.PressAKeyToReboot:             db      'Press a key to reboot.', 00Ah, 00Dh
.PressAKeyToReboot_Length:      equ     $-.PressAKeyToReboot

ErrorCode:
.100:                           db      '100'
.101:                           db      '101'

BootmanagerFileName:            db      'BOOTMAN SYS'




;-----------------------------------------------------------------------------
; Boot Sector variables.
;-----------------------------------------------------------------------------
; These variables must be placed as the last bytes in the boot sector.

; Some runtime variables.
DataStartLBA:           dd      000000000h
PartitionStartLBA:      dd      000000000h
FATStartLBA:            dd      000000000h
RootDirectoryStartLBA:  dd      000000000h

Max_Cylinder:           dd      000h
Max_Head:               dd      000h
Max_Sector:             dd      000h

CurrentCluster:         dd      000h

Partition:
.Drive:                 db      080h
.CylSec:                dw      00001h
.Head:                  db      001h

; Some assemble time variables.
Address:
.BootSector:            equ     08000h
.FAT_Area:              equ     Address.BootSector+0200h
.RootDirectory:         equ     Address.FAT_Area+0200h

FAT_Area:
.FirstEntry:            dd      000h
.LastEntry:             dd      000h
.SizeInSectors:         equ     001h
.BytesPerEntry:         dd      002h
.NumberOfEntries:       dd      0100h
RootDirectory:
.BytesPerEntry:         dd      020h

CurrentSegment:         dd      00000h

Program:
.Segment:               dw      09000h
.Start:                 dd      000h
.Pointer:               dd      000h
